library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

use work.ipbus.all;
use work.ipbus_reg_types.all;

use work.emp_data_types.all;
use work.emp_project_decl.all;
use work.emp_device_decl.all;
use work.emp_ttc_decl.all;

use work.ipbus_decode_emp_payload.all;

use work.dtc_link_maps.all;
use work.front_end_data_types.all;
use work.gbt_module_constants.all;
use work.mprocessor_constants.all;
use work.dtc_data_types.all;
use work.dtc_constants.all;


entity emp_payload is
port(
    --- Input Ports ---
    clk_p            : in  std_logic;
    clk40            : in  std_logic := '0';
    clk_payload      : in  std_logic_vector(2 downto 0);
    rst_payload      : in  std_logic_vector(2 downto 0);
    rst_loc          : in  std_logic_vector(N_REGION - 1 downto 0);
    clken_loc        : in  std_logic_vector(N_REGION - 1 downto 0);
    ctrs             : in  ttc_stuff_array;
    ttc_cmd          : in  ttc_cmd_t;
    d                : in  ldata(4 * N_REGION - 1 downto 0);
    --- Output Ports ---
    bc0              : out std_logic;
    gpio             : out std_logic_vector(29 downto 0);
    gpio_en          : out std_logic_vector(29 downto 0);
    q                : out ldata(4 * N_REGION - 1 downto 0);
    --- IPBus Ports ---
    clk              : in  std_logic;
    rst              : in  std_logic;
    ipb_in           : in  ipb_wbus;
    ipb_out          : out ipb_rbus;
    -- KH
    gbe_q            : out ldata := (others => LWORD_NULL);
    gbe_backpressure : in  std_logic_vector(1 downto 0)
);
end emp_payload;

architecture rtl of emp_payload is


    -- IPBus fabric

    signal ipb_to_slaves                : ipb_wbus_array(N_SLAVES - 1 downto 0);
    signal ipb_from_slaves              : ipb_rbus_array(N_SLAVES - 1 downto 0);

    signal channel_sel                  : ipb_reg_v(0 downto 0);
    signal mproc_channel_sel            : ipb_reg_v(0 downto 0);
    signal link_aggr_control            : ipb_reg_v(0 downto 0);


    -- FE data extraction and monitoring

    signal ipb_chain                    : ipbdc_bus_array(cNumberOfFEModules downto 0);
    signal stubs                        : ldata(cNumberOfFEModules - 1 downto 0);

    signal linksout                     : ldata(cNumberOfFEModules - 1 downto 0);
    signal linkor                       : lword                                    := LWORD_NULL;

    -- Histogrammer

    constant N_CTRL_HIST                : integer := 2;
    constant N_STAT_HIST                : integer := 2;

    constant bin_width                  : integer := 32;
    constant input_width                : integer := 9;

    signal status_registers             : ipb_reg_v(N_STAT_HIST - 1 downto 0) := (others => (others => '0'));
    signal control_registers            : ipb_reg_v(N_CTRL_HIST - 1 downto 0) := (others => (others => '0'));

    signal trigger_window_lower         : std_logic_vector(31 downto 0)            := (others => '0');
    signal trigger_window_upper         : std_logic_vector(3 downto 0)             := (others => '0');
    signal trigger_window               : std_logic_vector(36 - 1 downto 0)        := X"0ffffffff";
    signal max_value0, max_value1       : std_logic_vector(bin_width - 1 downto 0) := (others => '0');
    signal histogram_reset              : std_logic                                := '0';
    signal hist_l0_0_stub, hist_l0_1_stub       : lword                                    := LWORD_NULL;
    signal hist_l1_0_stub, hist_l1_1_stub       : lword                                    := LWORD_NULL;
    signal hist_l2_0_stub, hist_l2_1_stub       : lword                                    := LWORD_NULL;
    signal hist_l3_0_stub, hist_l3_1_stub       : lword                                    := LWORD_NULL;
    signal hist_l4_0_stub, hist_l4_1_stub       : lword                                    := LWORD_NULL;
    signal hist_l5_0_stub, hist_l5_1_stub       : lword                                    := LWORD_NULL;
    signal histogram_sel                : integer                                  := 0;
    signal histogram_enable             : std_logic                                := '0';
    signal header_start_array           : tHeaderStartArray                        := (others => (others => '0'));
    signal eth_link_out                 : ldata(1 downto 0)                        := (others => LWORD_NULL);

    signal header_array                 : tCICHeaderArray(cNumberOfFEModules * cNumberOfCICs - 1 downto 0) := (others => ('0', (others => '0'), (others => '0'), (others => '0')));

    -- MProcessor
    constant cNumberOfMProcessors       : integer                                  := 1;
    signal mprocessor_ipb_chain         : ipbdc_bus_array(cNumberOfMProcessors downto 0);
    signal srst                         : std_logic                                := '0';
    signal ttc_bc0, ttc_oc0             : std_logic                                := '0';

    signal orbit_counter                : unsigned(31 downto 0)                    := to_unsigned(0   , 32);
    signal bunch_counter                : unsigned(11 downto 0)                    := to_unsigned(3564, 12);

    -- Daqpath

    signal daq_read                   : tDaqFlagLinkArray(cNumberOfFEModules - 1 downto 0);
    signal daq_data                   : tDaqDataLinkArray(cNumberOfFEModules - 1 downto 0);
    signal daq_empty                  : tDaqFlagLinkArray(cNumberOfFEModules - 1 downto 0);

begin
  
--==============================--
-- IPBus fabric
--==============================--


--==============================--
fabric: entity work.ipbus_fabric_sel
--==============================--
generic map(
    NSLV            => N_SLAVES,
    SEL_WIDTH       => IPBUS_SEL_WIDTH
)
port map(
    ipb_in          => ipb_in,
    ipb_out         => ipb_out,
    sel             => ipbus_sel_emp_payload(ipb_in.ipb_addr),
    ipb_to_slaves   => ipb_to_slaves,
    ipb_from_slaves => ipb_from_slaves
);


--==============================--
fe_channel_ctrl: entity work.ipbus_reg_v
--==============================--
generic map(
    N_REG           => 1
)
port map(
    clk             => clk,
    reset           => rst,
    ipbus_in        => ipb_to_slaves(N_SLV_FE),
    ipbus_out       => ipb_from_slaves(N_SLV_FE),
    q               => channel_sel,
    qmask           => (0 => X"0000007f")
);

--==============================--
mproc_channel_ctrl: entity work.ipbus_reg_v
--==============================--
generic map(
    N_REG           => 1
)
port map(
    clk             => clk,
    reset           => rst,
    ipbus_in        => ipb_to_slaves(N_SLV_MPROC_CTRL),
    ipbus_out       => ipb_from_slaves(N_SLV_MPROC_CTRL),
    q               => mproc_channel_sel,
    qmask           => (0 => X"000000ff")
);


--==============================--
channel_select: entity work.ipbus_dc_fabric_sel
--==============================--
generic map(
    SEL_WIDTH       => 7
)
port map(
    clk             => clk,
    rst             => rst,
    sel             => channel_sel(0)(6 downto 0),
    ipb_in          => ipb_to_slaves(N_SLV_FE_CHAN),
    ipb_out         => ipb_from_slaves(N_SLV_FE_CHAN),
    ipbdc_out       => ipb_chain(0),
    ipbdc_in        => ipb_chain(cNumberOfFEModules)
);



--==============================--
-- FE data extraction and monitoring
--==============================--


--==============================--
genLinkInterface: for i in 0 to cNumberOfFEModules - 1 generate
--==============================--

    signal ipb_to_channel   : ipb_wbus;
    signal ipb_from_channel : ipb_rbus;

begin

    --==============================--
    channel_node: entity work.ipbus_dc_node
    --==============================--
    generic map(
        I_SLV       => i,
        SEL_WIDTH   => 7,
        PIPELINE    => false
    )
    port map(
        clk         => clk,
        rst         => rst,
        ipb_out     => ipb_to_channel,
        ipb_in      => ipb_from_channel,
        ipbdc_in    => ipb_chain(i),
        ipbdc_out   => ipb_chain(i + 1)
    );

    --==============================--
	LinkInterfaceInstance: entity work.LinkInterface
    --==============================--
    generic map (
        module_type => "2S"
    )
    port map (
        --- Input Ports ---
        clk_p               => clk_p,
        link_in             => d(cDTCInputLinkMap(i)),
        daq_read            => daq_read(i),
        --- Output Ports ---
        link_out            => linksout(i),
        stub_out            => stubs(i),
        daq_out             => daq_data(i),
        daq_empty           => daq_empty(i),
        header_out  => header_array(cNumberOfCICs * i + cNumberOfCICs - 1 downto cNumberOfCICs * i),
        --- IPBus Ports ---
        clk                 => clk,
        rst                 => rst,
        ipb_in              => ipb_to_channel,
        ipb_out             => ipb_from_channel,
        --- Debug Ports ---
        debug_header_start => header_start_array(i)
    );
end generate;


linkor.data(63 downto 0) <= linksout(0).data(63 downto 0) or linksout(1).data(63 downto 0) or linksout(2).data(63 downto 0) or linksout(3).data(63 downto 0) or linksout(4).data(63 downto 0) or linksout(5).data(63 downto 0);

--==============================--
genHack: for i in 0 to cNumberOfFEModules - 1 generate
--==============================--
begin
	q(cDTCInputLinkMap(i)).data <= linkor.data(63 downto 1) & ttc_bc0;
    q(cDTCInputLinkMap(i)).valid <= linkor.valid;
    q(cDTCInputLinkMap(i)).start <= linkor.start;
    q(cDTCInputLinkMap(i)).strobe <= linkor.strobe;
end generate;


--==============================--
-- L1 Data Aggregator
--==============================--

--==============================--
L1DataAggregator : entity work.L1DataAggregator
--==============================--
port map (
    --- Input Ports ---
    clk_p   => clk_p,
    daq_in  => daq_data,
    empty   => daq_empty,
    --- Output Ports ---
    read    => daq_read,
    --- IPBus Ports ---
    clk     => clk,
    rst     => rst,
    ipb_in  => ipb_to_slaves(N_SLV_BE_DAQ),
    ipb_out => ipb_from_slaves(N_SLV_BE_DAQ)
);


--==============================--
-- Histogrammer
--==============================--


--==============================--
histogram_ctrl: entity work.ipbus_ctrlreg_v
--==============================--
generic map(
    N_CTRL            => N_CTRL_HIST,
    N_STAT            => N_STAT_HIST
)
port map(
    clk               => clk,
    reset             => rst,
    ipbus_in          => ipb_to_slaves(N_SLV_CSR),
    ipbus_out         => ipb_from_slaves(N_SLV_CSR),
    d                 => status_registers,
    q		          => control_registers
);

status_registers(0)(bin_width - 1 downto 0)   <= max_value0;
status_registers(1)(bin_width - 1 downto 0)   <= max_value1;


trigger_window_lower  <= control_registers(0);
trigger_window_upper  <= control_registers(1)(3 downto 0);
histogram_enable      <= control_registers(1)(4);
histogram_sel         <= to_integer(unsigned(control_registers(1)(15 downto 5)));
trigger_window        <= trigger_window_upper & trigger_window_lower;


--==============================--
pHistogram: process(clk_p)
--==============================--
begin
    if rising_edge(clk_p) then
        if stubs(0).valid = '1' and histogram_enable = '1' then
            if stubs(0).data(50) = '0' then
                hist_l0_0_stub <= stubs(0);
                hist_l0_1_stub <= LWORD_NULL;
            else
                hist_l0_0_stub <= LWORD_NULL;
                hist_l0_1_stub <= stubs(0);
            end if;
        else
            hist_l0_0_stub <= LWORD_NULL;
            hist_l0_1_stub <= LWORD_NULL;
        end if;
        if stubs(1).valid = '1' and histogram_enable = '1' then
            if stubs(1).data(50) = '0' then
                hist_l1_0_stub <= stubs(1);
                hist_l1_1_stub <= LWORD_NULL;
            else
                hist_l1_0_stub <= LWORD_NULL;
                hist_l1_1_stub <= stubs(1);
            end if;
        else
            hist_l1_0_stub <= LWORD_NULL;
            hist_l1_1_stub <= LWORD_NULL;
        end if;
        if stubs(2).valid = '1' and histogram_enable = '1' then
            if stubs(2).data(50) = '0' then
                hist_l2_0_stub <= stubs(2);
                hist_l2_1_stub <= LWORD_NULL;
            else
                hist_l2_0_stub <= LWORD_NULL;
                hist_l2_1_stub <= stubs(2);
            end if;
        else
            hist_l2_0_stub <= LWORD_NULL;
            hist_l2_1_stub <= LWORD_NULL;
        end if;
        if stubs(3).valid = '1' and histogram_enable = '1' then
            if stubs(3).data(50) = '0' then
                hist_l3_0_stub <= stubs(3);
                hist_l3_1_stub <= LWORD_NULL;
            else
                hist_l3_0_stub <= LWORD_NULL;
                hist_l3_1_stub <= stubs(3);
            end if;
        else
            hist_l3_0_stub <= LWORD_NULL;
            hist_l3_1_stub <= LWORD_NULL;
        end if;
        if stubs(4).valid = '1' and histogram_enable = '1' then
            if stubs(4).data(50) = '0' then
                hist_l4_0_stub <= stubs(4);
                hist_l4_1_stub <= LWORD_NULL;
            else
                hist_l4_0_stub <= LWORD_NULL;
                hist_l4_1_stub <= stubs(4);
            end if;
        else
            hist_l4_0_stub <= LWORD_NULL;
            hist_l4_1_stub <= LWORD_NULL;
        end if;
        if stubs(5).valid = '1' and histogram_enable = '1' then
            if stubs(5).data(50) = '0' then
                hist_l5_0_stub <= stubs(5);
                hist_l5_1_stub <= LWORD_NULL;
            else
                hist_l5_0_stub <= LWORD_NULL;
                hist_l5_1_stub <= stubs(5);
            end if;
        else
            hist_l5_0_stub <= LWORD_NULL;
            hist_l5_1_stub <= LWORD_NULL;
        end if;
    end if;
end process;


--==============================--
HistogramInstanceL0CIC0: entity work.IPBusHistogram
--==============================--
generic map(
    input_width       => input_width,
    bin_width         => bin_width,
    data_offset       => 10
)
port map(
    --- Input Ports ---
    clk_p             => clk_p,
    data_in           => hist_l0_0_stub,
    histogram_reset   => histogram_reset,
    --- Output Ports ---
    max_bin_value     => max_value0,
    --- IPBus Ports ---
    clk               => clk,
    rst               => rst,
    ipb_in            => ipb_to_slaves(N_SLV_MEM1),
    ipb_out           => ipb_from_slaves(N_SLV_MEM1)
);


--==============================--
HistogramInstanceL0CIC1: entity work.IPBusHistogram
--==============================--
generic map(
    input_width       => input_width,
    bin_width         => bin_width,
    data_offset       => 10
)
port map(
    --- Input Ports ---
    clk_p             => clk_p,
    data_in           => hist_l0_1_stub,
    histogram_reset   => histogram_reset,
    --- Output Ports ---
    max_bin_value     => max_value1,
    --- IPBus Ports ---
    clk               => clk,
    rst               => rst,
    ipb_in            => ipb_to_slaves(N_SLV_MEM2),
    ipb_out           => ipb_from_slaves(N_SLV_MEM2)
);


--==============================--
HistogramInstanceL1CIC0: entity work.IPBusHistogram
--==============================--
generic map(
    input_width       => input_width,
    bin_width         => bin_width,
    data_offset       => 10
)
port map(
    --- Input Ports ---
    clk_p             => clk_p,
    data_in           => hist_l1_0_stub,
    histogram_reset   => histogram_reset,
    --- Output Ports ---
    --max_bin_value     => max_value0,
    --- IPBus Ports ---
    clk               => clk,
    rst               => rst,
    ipb_in            => ipb_to_slaves(N_SLV_MEM3),
    ipb_out           => ipb_from_slaves(N_SLV_MEM3)
);


--==============================--
HistogramInstanceL1CIC1: entity work.IPBusHistogram
--==============================--
generic map(
    input_width       => input_width,
    bin_width         => bin_width,
    data_offset       => 10
)
port map(
    --- Input Ports ---
    clk_p             => clk_p,
    data_in           => hist_l1_1_stub,
    histogram_reset   => histogram_reset,
    --- Output Ports ---
    --max_bin_value     => max_value1,
    --- IPBus Ports ---
    clk               => clk,
    rst               => rst,
    ipb_in            => ipb_to_slaves(N_SLV_MEM4),
    ipb_out           => ipb_from_slaves(N_SLV_MEM4)
);


--==============================--
HistogramInstanceL2CIC0: entity work.IPBusHistogram
--==============================--
generic map(
    input_width       => input_width,
    bin_width         => bin_width,
    data_offset       => 10
)
port map(
    --- Input Ports ---
    clk_p             => clk_p,
    data_in           => hist_l2_0_stub,
    histogram_reset   => histogram_reset,
    --- Output Ports ---
    --max_bin_value     => max_value0,
    --- IPBus Ports ---
    clk               => clk,
    rst               => rst,
    ipb_in            => ipb_to_slaves(N_SLV_MEM5),
    ipb_out           => ipb_from_slaves(N_SLV_MEM5)
);


--==============================--
HistogramInstanceL2CIC1: entity work.IPBusHistogram
--==============================--
generic map(
    input_width       => input_width,
    bin_width         => bin_width,
    data_offset       => 10
)
port map(
    --- Input Ports ---
    clk_p             => clk_p,
    data_in           => hist_l2_1_stub,
    histogram_reset   => histogram_reset,
    --- Output Ports ---
    --max_bin_value     => max_value1,
    --- IPBus Ports ---
    clk               => clk,
    rst               => rst,
    ipb_in            => ipb_to_slaves(N_SLV_MEM6),
    ipb_out           => ipb_from_slaves(N_SLV_MEM6)
);


--==============================--
HistogramInstanceL3CIC0: entity work.IPBusHistogram
--==============================--
generic map(
    input_width       => input_width,
    bin_width         => bin_width,
    data_offset       => 10
)
port map(
    --- Input Ports ---
    clk_p             => clk_p,
    data_in           => hist_l3_0_stub,
    histogram_reset   => histogram_reset,
    --- Output Ports ---
    --max_bin_value     => max_value0,
    --- IPBus Ports ---
    clk               => clk,
    rst               => rst,
    ipb_in            => ipb_to_slaves(N_SLV_MEM7),
    ipb_out           => ipb_from_slaves(N_SLV_MEM7)
);


--==============================--
HistogramInstanceL3CIC1: entity work.IPBusHistogram
--==============================--
generic map(
    input_width       => input_width,
    bin_width         => bin_width,
    data_offset       => 10
)
port map(
    --- Input Ports ---
    clk_p             => clk_p,
    data_in           => hist_l3_1_stub,
    histogram_reset   => histogram_reset,
    --- Output Ports ---
    --max_bin_value     => max_value1,
    --- IPBus Ports ---
    clk               => clk,
    rst               => rst,
    ipb_in            => ipb_to_slaves(N_SLV_MEM8),
    ipb_out           => ipb_from_slaves(N_SLV_MEM8)
);


--==============================--
HistogramInstanceL4CIC0: entity work.IPBusHistogram
--==============================--
generic map(
    input_width       => input_width,
    bin_width         => bin_width,
    data_offset       => 10
)
port map(
    --- Input Ports ---
    clk_p             => clk_p,
    data_in           => hist_l4_0_stub,
    histogram_reset   => histogram_reset,
    --- Output Ports ---
    --max_bin_value     => max_value0,
    --- IPBus Ports ---
    clk               => clk,
    rst               => rst,
    ipb_in            => ipb_to_slaves(N_SLV_MEM9),
    ipb_out           => ipb_from_slaves(N_SLV_MEM9)
);


--==============================--
HistogramInstanceL4CIC1: entity work.IPBusHistogram
--==============================--
generic map(
    input_width       => input_width,
    bin_width         => bin_width,
    data_offset       => 10
)
port map(
    --- Input Ports ---
    clk_p             => clk_p,
    data_in           => hist_l4_1_stub,
    histogram_reset   => histogram_reset,
    --- Output Ports ---
    --max_bin_value     => max_value1,
    --- IPBus Ports ---
    clk               => clk,
    rst               => rst,
    ipb_in            => ipb_to_slaves(N_SLV_MEM10),
    ipb_out           => ipb_from_slaves(N_SLV_MEM10)
);

--==============================--
HistogramInstanceL5CIC0: entity work.IPBusHistogram
--==============================--
generic map(
    input_width       => input_width,
    bin_width         => bin_width,
    data_offset       => 10
)
port map(
    --- Input Ports ---
    clk_p             => clk_p,
    data_in           => hist_l5_0_stub,
    histogram_reset   => histogram_reset,
    --- Output Ports ---
    --max_bin_value     => max_value0,
    --- IPBus Ports ---
    clk               => clk,
    rst               => rst,
    ipb_in            => ipb_to_slaves(N_SLV_MEM11),
    ipb_out           => ipb_from_slaves(N_SLV_MEM11)
);


--==============================--
HistogramInstanceL5CIC1: entity work.IPBusHistogram
--==============================--
generic map(
    input_width       => input_width,
    bin_width         => bin_width,
    data_offset       => 10
)
port map(
    --- Input Ports ---
    clk_p             => clk_p,
    data_in           => hist_l5_1_stub,
    histogram_reset   => histogram_reset,
    --- Output Ports ---
    --max_bin_value     => max_value1,
    --- IPBus Ports ---
    clk               => clk,
    rst               => rst,
    ipb_in            => ipb_to_slaves(N_SLV_MEM12),
    ipb_out           => ipb_from_slaves(N_SLV_MEM12)
);

--==============================--
HistogramResetterInstance: entity work.HistogramResetter
--==============================--
port map(
    --- Input Ports ---
    clk_p             => clk_p,
    trigger_window    => trigger_window,
    --- Output Ports ---
    histogram_reset   => histogram_reset
);



--==============================--
-- Link Aggregation and Combination
--==============================--

--==============================--
mprocessor_channel_select: entity work.ipbus_dc_fabric_sel
--==============================--
generic map(
    SEL_WIDTH       => 7
)
port map(
    clk             => clk,
    rst             => rst,
    sel             => mproc_channel_sel(0)(6 downto 0),
    ipb_in          => ipb_to_slaves(N_SLV_MPROCESSOR),
    ipb_out         => ipb_from_slaves(N_SLV_MPROCESSOR),
    ipbdc_out       => mprocessor_ipb_chain(0),
    ipbdc_in        => mprocessor_ipb_chain(cNumberOfMProcessors)
);


--==============================-
genMProcessors : for i in 0 to cNumberOfMProcessors - 1 generate
--==============================-

    signal ipb_to_channel        : ipb_wbus;
    signal ipb_from_channel      : ipb_rbus;
    constant cNumberofInputLinks : integer := 6;

begin

    --==============================--
    channel_node: entity work.ipbus_dc_node
    --==============================--
    generic map(
        I_SLV       => i,
        SEL_WIDTH   => 7,
        PIPELINE    => false
    )
    port map(
        clk         => clk,
        rst         => rst,
        ipb_out     => ipb_to_channel,
        ipb_in      => ipb_from_channel,
        ipbdc_in    => mprocessor_ipb_chain(i),
        ipbdc_out   => mprocessor_ipb_chain(i + 1)
    );

    --==============================--
    MProcessorInstance: entity work.MProcessor
    --==============================--
    generic map(
        N_LINKS => cNumberofInputLinks
    )
    port map(
        --- Input Ports ---
        clk_p              => clk_p,
        links_in           => stubs(cNumberofInputLinks*i + (cNumberofInputLinks - 1) downto cNumberofInputLinks*i),
        header_in          => header_array,
        header_start_array => header_start_array,
        gbe_backpressure   => gbe_backpressure(i),
        srst               => srst,
        --- Output Ports ---
        link_out           => eth_link_out(i),
        --- IPBus Ports ---
        ipb_clk            => clk,
        ipb_rst            => rst,
        ipb_in             => ipb_to_channel,
        ipb_out            => ipb_from_channel
    );
end generate genMProcessors;


--==============================--
pTTC: process(clk40)
--==============================--
begin
    if rising_edge(clk40) then

      ttc_bc0 <= '0';
      ttc_oc0 <= '0';

      if ttc_cmd = TTC_BCMD_BC0 then
        orbit_counter <= orbit_counter + 1;
        bunch_counter <= (others => '0');
        ttc_bc0 <= '1';

      elsif ttc_cmd = TTC_BCMD_OC0 then
        orbit_counter <= (others => '0');
        bunch_counter <= to_unsigned(3564, bunch_counter'length);
        ttc_oc0 <= '1';

      elsif bunch_counter < 3564 then
        bunch_counter <= bunch_counter + 1;        
      end if;

    end if;
end process;

srst <= mproc_channel_sel(0)(7) or ttc_oc0;

q(20).valid  <= '1';
q(20).data   <= "0000" & std_logic_vector(bunch_counter) & "0000000000000000" & std_logic_vector(orbit_counter);
q(20).start  <= '1';
q(20).strobe <= '1';


--==============================--


gbe_q <= eth_link_out;
-- q(24).valid  <= '1';
-- q(24).start  <= '1';
-- q(24).strobe <= '1';
-- q(24).data   <= gbe_backpressure(0) & eth_link_out(0).data(62 downto 0);
-- q(25).valid  <= '1';
-- q(25).start  <= '1';
-- q(25).strobe <= '1';
-- q(25).data   <= gbe_backpressure(1) & eth_link_out(1).data(62 downto 0);

bc0     <= '0';
gpio    <= (others => '0');
gpio_en <= (others => '0');

end rtl;
